"use strict";

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require("babel-runtime/core-js/json/stringify");

var _stringify2 = _interopRequireDefault(_stringify);

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const fetch = require(`isomorphic-fetch`);
const crypto = require(`crypto`);
const glob = require(`globby`);
const camelCase = require(`camelcase`);
const matter = require(`front-matter`);
const fs = require(`fs-extra`);
const PromisePool = require(`es6-promise-pool`);

const regStart = /[_a-zA-Z]/;

exports.sourceNodes = (() => {
	var _ref = (0, _asyncToGenerator3.default)(function* ({ boundActionCreators }, options) {
		options = (0, _assign2.default)({
			ids: [],
			markdownPath: false,
			apiKey: process.env.SALSIFY_API_KEY,
			org: process.env.SALSIFY_ORG,
			concurrency: 10,
			types: [],
			media: []
		}, options);

		if (!options.apiKey) {
			console.log(`No API key provided`);
			return;
		}

		if (!options.org) {
			console.log(`No Org provided`);
			return;
		}

		const url = `https://app.salsify.com/api/v1/orgs/${options.org}/products/`;
		const { createNode } = boundActionCreators;

		if (options.markdownPath) {
			let idsArrays = yield getIdsFromMarkdown(options.path);
			idsArrays.forEach(function (idArray) {
				idArray.forEach(function (id) {
					if (options.ids.indexOf(id) !== -1) return;
					options.ids.push(id);
				});
			});
		}

		/**
    * Fetch data from salsify
    * 
    * @param {string} url Url of Salsify API
    * @param {string} id ID of the product to fetch
    * @param {Object} options gatsby-config options
    * @returns {Promise} Promise chain
    */
		const salsifyFetcher = function (url, id, options) {
			return fetch(`${url}${id}`, {
				method: `GET`,
				headers: {
					Authorization: `Bearer ${options.apiKey}`
				}
			}).then(function (res) {
				if (res && res.status == 200) {
					return res.json();
				}
				return {};
			}).then(function (res) {
				res = formatSalsifyObject(res);
				for (let i in options.types) {
					if (res[i]) {
						if (options.types[i] == `array` && typeof res[i] === `string`) {
							res[i] = [res[i]];
						}
					}
				}
				options.media.forEach(function (key) {
					if (res[key]) {
						if (typeof res[key] === `string`) {
							res[key] = findDigitalAsset(res[key], res);
						} else {
							res[key] = res[key].map(function (id) {
								return findDigitalAsset(id, res);
							});
						}
					}
				});
				return (0, _assign2.default)({
					id: id,
					parent: null,
					children: [],
					internal: {
						type: `SalsifyContent`,
						contentDigest: crypto.createHash(`md5`).update((0, _stringify2.default)(res)).digest(`hex`)
					}
				}, res);
			}).then(function (datum) {
				createNode(datum);
			}).catch(function (error) {
				throw error;
			});
		};

		const generatePromises = function* () {
			for (let i = 0; i < options.ids.length; i++) {
				yield salsifyFetcher(url, options.ids[i], options);
			}
		};

		const promiseIterator = generatePromises();
		const pool = new PromisePool(promiseIterator, options.concurrency);
		yield pool.start().then(function () {
			return console.log(`Salsify Source - Done!`);
		});
		return;
	});

	return function (_x, _x2) {
		return _ref.apply(this, arguments);
	};
})();

function findDigitalAsset(id, res) {
	const arr = res[`salsify:digitalAssets`] || [];
	for (let i = 0; i < arr.length; i++) {
		if (arr[i][`salsify:id`] === id) {
			let obj = arr[i];
			let newObj = {};
			for (let i in obj) {
				newObj[i.replace(`salsify:`, ``)] = obj[i];
			}
			// Force HTTPS
			if (newObj.url && newObj.url.indexOf(`http:`) === 0) {
				newObj.url = newObj.url.replace(`http:`, `https:`);
			}
			return newObj;
		}
	}
}

function formatSalsifyObject(obj) {
	const newObj = {};
	for (let i in obj) {
		let camelKey = camelCase(i);
		if (camelKey.charAt(0).match(regStart)) {
			newObj[camelKey] = obj[i];
		} else {
			newObj[`_${camelKey}`] = obj[i];
		}
	}
	return newObj;
}

function getIdsFromMarkdown(path) {
	path = `${path}/**/*.md`;
	return glob(path).then(paths => {
		return _promise2.default.all(paths.map(path => {
			return fs.readFile(path).then(data => {
				let updated = [];
				data = data.toString();
				data = matter(data);
				if (data.attributes.variants) {
					data.attributes.variants.forEach(variant => {
						updated.push(variant.id.toUpperCase());
					});
				}
				if (data.attributes.id) {
					updated.push(data.attributes.id.toUpperCase());
				}
				return updated;
			});
		}));
	}).catch(console.error);
}